"""
🔗 G4F

"""

import importlib
from time import sleep

from loguru import logger

from .client import AIClient


class G4fHandler(AIClient):
    """
    MyLLM class for G4F

    """

    def __init__(self, **kwargs):
        """
        Initialize the MyLLM object

        Args:
            None
        """
        super().__init__(**kwargs)
        logger.debug("G4F provider initializing")
        self.llm_provider = kwargs.get("llm_provider", None)
        if self.enabled and self.llm_provider:
            provider_module_name = self.llm_provider
            provider_module = importlib.import_module(provider_module_name)
            provider_class = getattr(
                provider_module, provider_module_name.split(".")[-1]
            )
            self.provider = provider_class()
            self.client = self.provider
            logger.info("Provider {} initialized ", self.provider)

    async def chat(self, prompt):
        """
        Asynchronously chats with the user.

        Args:
            prompt (str): The prompt message from the user.

        Returns:
            str: The response from the conversation model.
        """
        self.conversation.add_message("user", prompt)
        response = None
        try:
            response = await self.provider.create_async(
                model=self.llm_model,
                messages=self.conversation.get_messages(),
            )
        except Exception as error:
            logger.error(error)
        sleep(self.timeout)

        logger.debug("response {}", response)
        if response:
            self.conversation.add_message("assistant", response)
            formatted_response = f"{self.llm_prefix} {response} {self.llm_suffix}"
            logger.debug("User: {}, AI: {}", prompt, response)
            return formatted_response



"""
🔗 OpenAI and LocalAI Support

via https://github.com/openai/openai-python
via https://localai.io

"""

from time import sleep

from openai import OpenAI

from .client import AIClient


class OpenaiHandler(AIClient):
    """
    MyLLM class for OpenAI and LocalAI

    """

    def __init__(self, **kwargs):
        """
        Initialize the object with the given keyword arguments.

        :param kwargs: keyword arguments
        :return: None
        """

        super().__init__(**kwargs)
        if self.enabled and self.llm_provider_key:
            self.llm_base_url = kwargs.get("llm_base_url", None)
            self.client = OpenAI(
                api_key=self.llm_provider_key, base_url=self.llm_base_url
            )

    async def chat(self, prompt):
        """
        Asynchronously chats with the client based on the given prompt.

        :param prompt: The prompt for the chat.
        :return: The response from the chat.

        """
        self.conversation.add_message("user", prompt)
        archived_messages = self.conversation.get_messages()

        response = self.client.chat.completions.create(
            model=self.llm_model,
            messages=archived_messages,
        )
        sleep(self.timeout)

        if response_content := response.choices[0].message.content:
            self.conversation.add_message("assistant", response_content)
            return f"{self.llm_prefix} {response_content}"
